/* Jeff Connelly
 * C4 L2 Extra Flexible Version
 * With Programmable Equation Evaluator
 */

#include <iostream.h>
#include <stdlib.h>
#include <apstring.h>
#include <iomanip.h>
#include <math.h>
#include <assert.h>

const double PI = 3.141592;

double ask_for_double(apstring prompt, int metric);
void new_function();

// A simple Reverse Polish Notation evaluation machine (C) 2002 Jeff Connelly.
// This function has to be before main(), because prototypes with default parameters 
// cause problems.
double rpn(const char* ops, 
		   double a = 0, 
		   double b = 0, 
		   double c = 0,
		   double d = 0,
		   double e = 0,
		   double f = 0)
{
	int oi;           // ops index
	char op;           // current operator
	double stack[10] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, } ; // stack
	int si = 0;       // stack index
	double x, y;

	assert(ops);

	cout << "Running " << ops << endl;

	for (oi = 0; oi < 20; oi++)
	{
		op = ops[oi];
		switch(op)
		{
		case '+': y = stack[--si]; x = stack[--si]; stack[si] = x + y;   si++; break;
		case '-': y = stack[--si]; x = stack[--si]; stack[si] = x - y;   si++; break;
		case '*': y = stack[--si]; x = stack[--si]; stack[si] = x * y;   si++; break;
		case '/': y = stack[--si]; x = stack[--si]; stack[si] = x / y;   si++; break;
		case '^': y = stack[--si];                  stack[si] = sqrt(y); si++; break;
		case 'a': stack[si] = a; si++; break;
		case 'b': stack[si] = b; si++; break;
		case 'c': stack[si] = c; si++; break;
		case 'd': stack[si] = d; si++; break;
		case 'e': stack[si] = e; si++; break;
		case 'f': stack[si] = f; si++; break;
		case 0:
		case '.': return stack[--si]; break; 
		case '=': // for debugging, show stack
			for (int i = 0; i < 20; i++)
			{
				cout << ((i == si) ? "*" : " ") << i << "\t" << stack[i] << endl;
			}
			break;
		}
	}
	return stack[si];
}

// This exception is thrown when there is a serious problem with user input
class BadUserException
{
public:
	BadUserException(apstring r) { reason = r; }
	apstring msg() { return reason; }
private:
	apstring reason;
};

char* fnames[] = 
{
	"quit",
	"NEW FUNCTION",
	"celsius -> fahrenheit",
	"fahrenheit -> celsius",
	"spherical radius",
	"hypotenuse of right triangle",
	0
};

char* functions[] =
{
	0,
	0,
	"ab/c*d+.",
	"ab-cd/*.",
	"ab/c*d*d*d*.",
	"aa*bb*+^."
};

char* fprompts1[] = 
{
	0,
	0,
	"Enter Celsius temperature for Fahrenheit: ",
	"Enter Fahrenheit temperature for Celsius: ",
	"Enter sphere radius: ",
	"Enter first leg of right triangle: ",
};

char* fprompts2[] =
{
	0,
	0,
	0,
	0,
	0,
	"Enter second leg of right triangle: ",
};

// function inputs, with -1 as first prompt, -2 as second
double inputs[6][5] =
{
	{ 0, -0, -0, 0, 0 },
	{ 0, -0, -0, 0, 0 },
	{ 9, 5, -1, 32, 0 },
	{ -1, 32, 5, 9, 0 },
	{ 4, 3, PI, -1, 0 },
	{ -1, -2, 0, 0, 0 },
};


int fmetrics[] = { 0, 0, 1, 1, 1 };

int main()
{
	double a, b, output;
	char runagain = 'y';
	int choice, max_choice;

	cout << "Functions" << endl << endl;
	
	do
	{
		for(max_choice = 0; fnames[max_choice]; max_choice++)
		{
			cout << max_choice << ") " << fnames[max_choice] << endl;
		}

		choice = (int)ask_for_double("Select function: ", 1);
		if (!choice || choice > max_choice)
			break;
		if (choice == 1)
			new_function();

		// prompt for user inputs, catching any errors
		a = b = 0;
		try
		{
			if (fprompts1[choice])
			{
				cout << "A) ";
				a = ask_for_double(fprompts1[choice], fmetrics[choice]);
			}
			if (fprompts2[choice])
			{
				cout << "B) ";
				b = ask_for_double(fprompts2[choice], fmetrics[choice]);
			}
		} catch(BadUserException exception) {
			cout << "Negative on non-negative '" << exception.msg() << "', restarting" << endl 
				 << endl;
			continue;
		}

		cout << "choice=" << choice << endl
			<< "a=" << a << endl
			<< "b=" << b << endl;

		// Process inputs
		double in[5];
		
		for (int i = 0; i <= 4; i++)
		{
			in[i] = inputs[choice][i];
			if (in[i] == -1)
			{
				in[i] = a;
			} else if (in[i] == -2) {
				in[i] = b;
			}
			cout << "input #" << i << " = " << in[i] << endl;
		}

		if (!functions[choice])
		{
			cout << "function not found" << endl;
			continue;
		}
		output = rpn(functions[choice], in[0], in[1], in[2], in[3], in[4]);

		cout << endl << "Output = " << output << endl;
	
		cin.ignore(80, '\n');
	} while(runagain == 'y' || runagain == 'Y');
	return 0;
}


// Read a positive double > 0 from the user with specified prompt
// Exception if value is negative and non-metric
// Loops until non-zero value
// apstring prompt:  string to prompt user with
// int metric     :  true if using to measure lengths
double ask_for_double(apstring prompt, int metric = 0)
{
	double your_input;
	apstring my_input;

	your_input = 0;

	// Loop until get valid user input
	while(!your_input)
	{
		cout << prompt;
		getline(cin, my_input);
		your_input = atof(my_input.c_str());

		// special case of 0 really equalling 0
		if (my_input == "0")
			return 0;

		// Negative values cause complete restart
		if (metric && your_input < 0)
		{
			throw BadUserException(prompt);
		}
		if (your_input)
			break;

		cout << "Please enter a positive nonzero real number" << endl << endl;
	}
	return your_input;
}

void new_function()
{
	apstring function;
	double in[5];
	double out;

	cout << "Custom Function Evaluator" << endl << endl;
	cout << "Pre-defined functions" << endl;
	for(int i = 2; functions[i]; i++)
	{
		cout << functions[i] << "\t" << fnames[i] << "\tinputs=[";
		for(int j = 0; j < 4; j++)
		{
			cout << inputs[i][j] << " ";
		}
		cout << "]" << endl;
	}
	cout << endl;

	do
	{
		cout << endl << "Enter function in RPN notation, q to quit: ";
		getline(cin, function);
		if (function[0] == 'q')
			break;
		cout << "Input inputs, terminate with 42\n";
		for(int i = 0; i < 5; i++)
		{
			cout << "Input #" << i << " = ";
			in[i] = ask_for_double("", 0);
			if (in[i] == 42)
				break;
		}

		out = rpn(function.c_str(), in[0], in[1], in[2], in[3], in[4]);

		cout << "Output = " << out << endl;
		cout << flush;
		cin.ignore(80, '\n');
	} while(1);
}