/* Jeff Connelly
 * C4 L2
 * Mixed Functions
 */

#include <iostream.h>
#include <stdlib.h>
#include <apstring.h>
#include <iomanip.h>
#include <math.h>

const double PI = 3.14159;

double temp_c2f(double c);
double temp_f2c(double f);
double right_triangle_hypotenuse(double leg1, double leg2);
double spherical_volume(double radius);

double ask_for_double(apstring prompt, int metric);

// This exception is thrown when there is a serious problem with user input
class BadUserException
{
public:
	BadUserException(apstring r) { reason = r; }
	apstring msg() { return reason; }
private:
	apstring reason;
};

int main()
{
	double c, f, r, leg1, leg2;
	char runagain = 'y';
	
	do
	{
		// prompt for user inputs, catching any errors
		try
		{
			c = ask_for_double("Enter Celsius temperature for Fahrenheit: ", 0);
			f = ask_for_double("Enter Fahrenheit temperature for Celsius: ", 0);
			r = ask_for_double("Enter sphere radius: ", 1);
			leg1 = ask_for_double("Enter first leg of right triangle: ", 1);
			leg2 = ask_for_double("Enter second leg of right triangle: ", 1);
		} catch(BadUserException exception) {
			cout << "Negative on non-negative '" << exception.msg() << "', restarting" << endl 
				 << endl;
			continue;
		}

		// calculate numbers from equations and user inputs
		cout << c << "'C = " << temp_c2f(c) << "'F" << endl;
		cout << f << "'F = " << temp_f2c(f) << "'C" << endl;
		cout << "Sphere radius = " << r << ", volume = " << spherical_volume(r) << endl;
		cout << "Right triangle, legs = " << leg1 << ", " << leg2 << ", hypotenuse = " <<
			right_triangle_hypotenuse(leg1, leg2) << endl;

		cout << setiosflags(ios::right | ios::fixed | ios::showpoint)
			 << setprecision(2);

		cout << "Run again? (y)";
		cin >> runagain;
		cin.ignore(80, '\n');
	} while(runagain == 'y' || runagain == 'Y');
	return 0;
}

// a^2 + b^2 = c^2
// (a^2 + b^2)^1/2 = (c^2)^1/2;
// c = (a^2 + b^2)^1/2
// RPN-> a a * b b * + sqrt
double right_triangle_hypotenuse(double a, double b)
{
	return sqrt(a * a + b * b);
}

// RPN-> 4 3 / PI * r * r * r *
double spherical_volume(double r)
{
	return 4. / 3. * PI * r * r * r;
}

// C = 5/9(F - 32)
// C = 5/9F - (32)5/9
// C + (32)5/9 = 5/9F
// * 9/5         * 9/5
// 9/5C + 32 = F
// F = 9/5C + 32
// RPN-> 9 5 / C * 32 +
double temp_c2f(double c)
{
	return (9. / 5. * c) + 32;
}

// C = 5/9(F - 32)
// RPN-> F 32 - 5 9 / *
double temp_f2c(double f)
{
	return 5. / 9. * (f - 32);
}

// Read a positive double > 0 from the user with specified prompt
// Exception if value is negative and non-metric
// Loops until non-zero value
// apstring prompt:  string to prompt user with
// int metric     :  true if using to measure lengths
double ask_for_double(apstring prompt, int metric = 0)
{
	double your_input;
	apstring my_input;

	your_input = 0;

	// Loop until get valid user input
	while(!your_input)
	{
		cout << prompt;
		getline(cin, my_input);
		your_input = atof(my_input.c_str());

		// Negative values cause complete restart
		if (metric && your_input < 0)
		{
			throw BadUserException(prompt);
		}
		if (your_input)
			break;

		cout << "Please enter a positive nonzero real number" << endl << endl;
	}
	return your_input;
}
