/* Jeff Connelly
 * C4 L3
 * Quadratic Equations
 */

#include <iostream.h>
#include <apstring.h>
#include <math.h>
#include <bool.h>
#include <stdlib.h>
#include <iomanip.h>

typedef struct tagQUADRATIC_COEFFICIENTS
{
	int a;
	int b;
	int c;
} QUADRATIC_COEFFICIENTS;

typedef struct tagSQ_ROOT
{
	int n;
	bool perfect; // if 1: value=n, elsif 0: value=sqrt(n)
	bool imag;    // if 1: i*n, elsif 0: n
	int s;        // sign: 1,-1
} SQ_ROOT;

typedef struct tagFRACTION
{
	int num;      // -b
	SQ_ROOT sq;   // +- sq
	int denom;    // /2a
} FRACTION;

typedef struct tagQUADRATIC_SOLUTION
{
	FRACTION s1;
	FRACTION s2;
} QUADRATIC_SOLUTION;


int ask_for_int(apstring prompt);
int determinate(int a, int b, int c);
QUADRATIC_SOLUTION solve_quadratic(int a, int b, int c);
QUADRATIC_COEFFICIENTS ask_for_3int();
void print_solution(FRACTION);
FRACTION reduce_fraction(FRACTION);
bool divisible(int num, int den);

int main()
{
	int a, b, c, d;
	QUADRATIC_SOLUTION solution;
	//QUADRATIC_COEFFICIENTS e;
	char ra;
	
	
	cout << "Algebraic Quadratic Equation Solving Program" << endl
		 << "Jeff Connelly (C) 2002" << endl
		 << "Capabilities: imaginary numbers, exact solutions, no reducing" << endl << endl;

	do
	{
		//cout << "ax^2 +\tbx +\tc";
		//e = ask_for_3int();
		a = ask_for_int("ax^2, a=");
		b = ask_for_int("bx^1, b=");
		c = ask_for_int("cx^0, c=");

		if ((d = determinate(a, b, c)) < 0)
		{
			cout << "Solutions: 2 imaginary, d=" << d << endl;
		} else if (d == 0) {
			cout << "Solutions: 1 real, d=" << d << endl;
		} else {
			cout << "Solutions: 2 real, d=" << d << endl;
		}

		solution = solve_quadratic(a, b, c);
		cout << "*** Solution 1 ***" << endl;
		solution.s1 = reduce_fraction(solution.s1);
		print_solution(solution.s1);
		cout << endl << endl;
		cout << "*** Solution 2 *** " << endl;
		print_solution(solution.s2);
		cout << endl << endl;

		cout << "Again? ";
		cin >> ra;
		cin.ignore(80, '\n');

	} while (!(ra != 'y'));

	return 0;

}

apstring n2aps(int n)
{
	apstring s;
	char* lp = new char[n & 0xFF];  // you can never have too much overkill, unless >0xFF

	itoa(n, lp, 10);
	s = lp;

	return s; // warning: this isn't freed automatically, and isn't manually either
}

apstring repeat(apstring ch, int times)
{
	apstring s;
	while(times--)
		s += ch;
	return s;
}

int MAX(int a, int b)
{
	return (a > b) ? a : b;
}

// Textual representation of a SQ_ROOT structure
apstring fmt_sqrt(SQ_ROOT sq)
{
	apstring s;

	s = (sq.s < 0 ? apstring("- ") : "+ "); // addition or subtraction
	if (sq.imag)
	{
		if (sq.perfect)
		{
			s += n2aps(sq.n) + apstring("i");
		} else {
			s += apstring("sqrt(") + n2aps(sq.n) + apstring(")i");
		}
	} else {
		if (sq.perfect)
		{
			s += n2aps(sq.n);
		} else {
			s += apstring("sqrt(") + n2aps(sq.n) + apstring(")");
		}
	}		
	//apstring(" sqrt(") + n2aps(sq.n) + apstring(")");
	return s;
}

// Solution is already stored as an equation, print it in equation and real form
void print_solution(FRACTION f)
{
	apstring numer;
	apstring denom;
	double approx;
	int width;

	denom = n2aps(f.denom);
	
	//cout << "Real, exact: " << endl;
	if (f.sq.perfect)   // (-b + -?x) / 2a
	{
		numer = n2aps(f.num + (f.sq.s) * f.sq.n);
		if (f.sq.imag)
			numer += "i";
	} else {
		numer = n2aps(f.num) + apstring(" ") + fmt_sqrt(f.sq);
	}
	width = MAX(numer.length(), denom.length());

	approx = (f.num + (f.sq.s * (f.sq.perfect ? f.sq.n : 
		sqrt(f.sq.n)))) / f.denom;

	if (int(approx) == approx)      // integer solution, not an approximate real
	{
		cout << "Real, exact: " << approx << endl;
	} else {                        // real, but non-integer solution, print exact values as well
		cout << numer << endl << repeat("-", width) << endl;
		cout << repeat(" ", (width - denom.length()) / 2) << denom << endl << endl;
		if (!f.sq.imag)
			cout << "Real, approximate: "
			<< setiosflags(ios::right | ios::fixed | ios::showpoint)
			<< setprecision(4) << approx << endl;
	}
}

	
SQ_ROOT square_root(int eger)
{
	SQ_ROOT r;

	if ((int)sqrt(eger) * (int)sqrt(eger) == eger) // perfect square
	{
		if (eger >= 0)
		{
			r.n = (int)sqrt(eger);   // no loss from double->int, checked above
			if ((int)sqrt(eger) != (double)sqrt(eger))
			{
				cout << "root " << eger << " error\n";
			}
			r.perfect = true;        // already square rooted
			r.imag = false;
		} else {
			r.imag = true;
			r.n = (int)sqrt(-eger);
		}
	} else {               // store the sqrt, its as close to the real thing as we can get
		r.n = eger;
		r.perfect = false; // saved
		if (eger < 0)
		{
			r.n = -eger;
			r.imag = true;
		} else
			r.imag = false;
	}
	return r;
}

QUADRATIC_SOLUTION solve_quadratic(int a, int b, int c)
{
	QUADRATIC_SOLUTION sol;
	int det;
	SQ_ROOT rd;

	det = determinate(a, b, c);
	rd  = square_root(det);
	
	sol.s1.num   = -b;
	sol.s1.sq    = rd;
	sol.s1.sq.s  = 1;
	sol.s1.denom = 2 * a;

	sol.s2.num   = -b;
	sol.s2.sq    = rd;
	sol.s2.sq.s  = -1;
	sol.s2.denom = 2 * a;

	return sol;
}


int determinate(int a, int b, int c)
{
	// b^2 - 4ac
	return (b * b) - (4 * a * c);
}

// Read 3 integers, separated by tabs. Incomplete
QUADRATIC_COEFFICIENTS ask_for_3int()
{
	apstring s;
	QUADRATIC_COEFFICIENTS e;

	do
	{
		getline(cin, s);

		e.a = atoi(s.c_str());
		e.b = atoi(strchr(s.c_str(), '\t'));
		e.c = atoi(strchr(strchr(s.c_str(), '\t') + 1, '\t'));

		/* no error checking
		if (!e.a && s != "0")
		{
			cout << "invalid 1st input '" << e.a << "'" << endl;
			continue;
		} else if (!e.b && (apstring)strchr(s.c_str(), '\t') != "0") {
			cout << "invalid 2nd input '" << e.b << "'" << endl;
			continue;
		} else if (!e.c && (apstring)strchr(strchr(s.c_str(), '\t') + 1, '\t') != "0") {
			cout << "invalid 3rd input '" << e.c << "'" << endl;
			continue;
		} */
		break;
	} while(1);
	return e;
}

// Reads a valid integer from cin
int ask_for_int(apstring prompt)
{
	int your_input;
	apstring my_input;

	your_input = 0;

	// Loop until get valid user input
	while(!your_input)
	{
		cout << prompt;
		getline(cin, my_input);
		your_input = atoi(my_input.c_str());

		// special case of 0 really equalling 0
		// caveat: 0.0, +0, -0, 0.00, etc. are not zero
		if (my_input == "0")
			return 0;
		if (your_input)
			break;

		cout << "Please enter an integer number" << endl << endl;
	}
	return your_input;
}

bool divisible(int a, int b)
{
	return (a / b) == ((float)a / (float)b);
}

// Reduce a fraction with integer numerator + square root / integer denominator
// Incomplete
FRACTION reduce_fraction(FRACTION in)
{
	FRACTION out;

	out = in;

	// Case 1. a +- sqrt(b)
	//         ------------
	//             c
	// divisble: a/c
	// divisble: b^2/c
	if (divisible(in.num, in.denom) && divisible(in.num, 
		(in.sq.perfect ? in.sq.n : in.sq.n * in.sq.n)))
	{
		cout << "num + denom div" << endl;
	}
	return out;
}

