/* Jeff Connelly
 * C6 L2 - Quiz
 * 01 Oct 2002
 */

#include <apstring.h>
#include <apvector.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#include "initvect.h"

// Question-Answer(s) pair
typedef struct tagQA
{
	apstring q;             // Question
	apvector<apstring> a;   // Answer array
	int correct;            // Correct answer, index to a[]
} QA;

QA make_qa(apstring, apvector<apstring>);
QA load_qa(int len, char* q, ...);
apvector<QA> load_all(int len, ...);
void randomize(apvector<QA>& a);
int give_test(apvector<QA> questions);
void report_card(int correct, int total);

// apvector_apstring(len, ...) for arbitrary-sized apvector<apstring> initialization
SETUP_INITVECT_MANUAL(apstring, apstring, char*);
SETUP_OUTVECT(apstring, "");

int main()
{
	apvector<QA> questions;
	int count_correct;
	int count_total;
	char runagain;


	system("mode con lines=50");       // Screen size 80x50

	cout << "C++ Chapter 6 Quiz\n";
	cout << "At any prompt, type ? for help. Good luck!\n\n";

	do
	{

		// Load questions and correct answers
		questions = load_all(5, 
			"What operators have the highest precedence in C++?", 
			"* / %", 
			"+ -", 
			"! (unary)- (cast) ++ -- sizeof",    // 0=2
			"< <= > >=", 
			"&& ||",

			"!(a || b) == ", 
			"!a && !b",                          // 1=0
			"!a || !b", 
			"(a || b) ! (b || a)", 
			"(a && b) || (b && a)", 
			"(a || b) && (a || b)",
  
			"Which functions are called in this code snippet? open() || abort()",
			"open() and abort()",
			"open(), and abort() if open() is true",
			"open() only",
			"abort() only",
			"open(), and abort() if open() is false", // 2=4
        
			"open() && read() is equivalent in program flow to:",
			"if (open()) read();",                   // 3=0
			"if (!open()) read();",
			"if (open()) !read();",
			"if (!open()) !read();",
			"open(); read();",
        
			"!(a > b) is equivalent to:",
			"!a > !b",
			"!a > b",
			"a <= b",                                // 4=2
			"!a <= !b",
			"a >= b"
			);
		questions[0].correct = 2;
		questions[1].correct = 0;
		questions[2].correct = 4;
		questions[3].correct = 0;
		questions[4].correct = 0;

		randomize(questions);                   // Randomize questions and possibly answers
		count_correct = give_test(questions);   // Give test, returning total number correct
		count_total = questions[0].a.length();  // Total number of questions, for denominator of score
		report_card(count_correct, count_total);// Show score
		cout << "Run again (y): ";
		cin >> runagain;
		cin.ignore(80, '\n');
	} while (runagain == 'y');

	system("pause");

	return 0;
}

// Display grade
void report_card(int correct, int total)
{
	int grade;

	cout << "Final Score: " << correct << "/" << total << " = ";
	grade = int((((double)(correct)) / total) * 100);
	cout << grade << "%" << endl;

	if (correct == 0)             // 0 correct is BSoD
	{
		cout << "Enjoy the BSoD" << endl;
		__asm { int 0x0 } // Recoverable
	} else if (correct == 1) {    // 1 correct=illegal operation
		cout << "1 correct is unacceptable" << endl;
		*((char*)0) = 0;
	} else if (correct < 0) { // Easter egg
		__asm { int 0x10 } // Real-mode BIOS interrupt, cant be called in pmode (hard crash)
	}
}

// Give test, returning answers correct
int give_test(apvector<QA> questions)
{
	// Iterate through questions in reverse order
	int i = questions.length();
	int count_correct = 0;
	int count_total = i;
	while(i--)
	{
		apstring str_user_answer;
		int user_answer = -1;

		cout << questions.length() - i << ". " << questions[i].q << "\n";
		cout << questions[i].a, "\n";

		while(!(user_answer >= 0) || !(user_answer <= 5))
		{
			cout << "Answer: ";
			getline(cin, str_user_answer);
			if (str_user_answer.length() == 0)
				continue;
			if (str_user_answer[0] == '?') // help
			{
				system("start http://www.cppreference.com/");
			}
			user_answer = tolower(str_user_answer.c_str()[0]) - 'a'; // A..E a..e
		}

		cout << "Entered: " << 
			char(user_answer + 'A') << "\t Correct: " << 
			char(questions[i].correct + 'A') << "\t";
		int correct = questions[i].correct;
		apvector<apstring> a = questions[i].a;

		if (user_answer == questions[i].correct)
		{
			cout << "Correct   ";
			++count_correct;
		} else if (user_answer == ('f' - 'a')) {   // "f" is not even a valid answer
			--count_correct;
		} else {
			cout << "Incorrect ";
		}
		cout << "\tScore: " << count_correct << "/" << 5 - i << " " <<
			(int)((double)count_correct / (5 - i) * 100) << "%";
		cout << "\n\n";
	}
	return count_correct;
}

QA make_qa(apstring q, apvector<apstring> a)
{
	QA ret;

	ret.q = q;
	ret.a = a;
	return ret;
}

// Loads questions and groups of 5 answers from argument list into an apvector<QA>
// Usage: load_all(len, q1, a, b, c, d, e, q2, a, b, c, d, e, q3, a, b, c, d, e, q4, a...)
// "a" is the correct answer
apvector<QA> load_all(int len, ...)
{
	apvector<QA> v;
	int pos = 0; 
	va_list marker; 
	
	v.resize(len); 
	
	va_start(marker, len); 
	while(--len != -1) 
	{ 
		//i = va_arg(marker, char*); 
		v[len].q = va_arg(marker, char*);
		v[len].a.resize(5);
		v[len].a[0] = va_arg(marker, char*);
		v[len].a[1] = va_arg(marker, char*);
		v[len].a[2] = va_arg(marker, char*);
		v[len].a[3] = va_arg(marker, char*);
		v[len].a[4] = va_arg(marker, char*);
		v[len].correct = 0;  // When shuffled, update
	} 
	va_end(marker); 
	return v; 
}

// Create a QA structure with v.q=q and v.a.length()=len-2
QA load_qa(int len, char* q...)
{ 
	QA v;
	int pos = 0; 
	apstring i; 
	va_list marker; 
	
	v.q = q;
	v.a.resize(len); 
	
	va_start(marker, q); 
	while(--len != -1) 
	{ 
		i = va_arg(marker, char*); 
		v.a[pos++] = i; 
	} 
	va_end(marker); 
	return v; 
} 

// Randomize quiz
void randomize(apvector<QA>& a)
{
	srand(time(0));
	int i;

	// Fisher-Yates-shuffle questions
	for (i = a.length(); i--; )
	{
		int j;
		QA temp;

		// Random number from 0..i+1
		j = int((double)rand() / RAND_MAX * i) + 1;
		if (i == j)
			continue;

		// Swap a[i] [j]
		temp = a[j];
		a[j] = a[i];
		a[i] = temp;
	}
	return;

	// Fisher-Yates-Shuffle answers
	for (i = a.length(); i--; )
	{
		int j;
		apstring temp;

		for (j = a[i].a.length(); j--; )
		{
			int k;

			k = int((double)rand() / RAND_MAX * a[i].a.length());
			if (j == k)  // Nothing to swap
				continue;

			// Swap answers
			temp = a[i].a[k];
			a[i].a[k] = a[i].a[j];
			a[i].a[j] = temp;

			// Initially correct is #0, now update it because it was moved around
			if (j == a[i].correct)
			{
				cout << "Question #" << i << ", " << a[i].q << endl;
				cout << "Old Correct: #" << a[i].correct << " = " << a[i].a[a[i].correct] << endl;
				cout << "j = correct, so swapping with k\n";

				a[i].correct = k;

				cout << "New Correct: #" << a[i].correct << " = " << a[i].a[a[i].correct] << endl;
				exit(0);
			}
		}
	}
}
