/* Jeff Connelly
 * 04 Nov 2002
 * Euclid's GCF Algorithm
 */

#include <iostream.h>     // cout, cin
#include <apstring.h>     // apstring
#include <math.h>         // ceil
#include <stdio.h>        // sprintf

int GCF(int m, int n);
int GCF_slow_iter(int m, int n);
int decimal_digits(int n);

int main()
{
	cout << "Euclid-Knuth Greatest Common Factor Algorithm Implementation" << endl << endl;
	cout << "Hit Control-C at any time to exit. Otherwise, enter two numbers." << endl;

	while(1)              // Loop until break condition (usually ^C or ^Break)
	{
		apstring str;
		int m, n, gcf;

		cout << "m,n= ";
		getline(cin, str);
		m = atoi(str.c_str());
		if (decimal_digits(m) + 1 > str.length())
		{
			cout << "Please enter m and n separated by an individual character." << endl;
			continue;
		}

		n = atoi(str.c_str() + decimal_digits(m) + 1);
		gcf = GCF(m, n);

		cout << "GCF(" << m << ", " << n << ") = " << gcf << "\t";
		if (gcf == -1 || gcf == +1)
			// Relatively Prime: greatest common divisor being unity
			cout << "(relatively prime)";
		else if (gcf == 0)
			cout << "(infinity encountered)";
		cout << endl;
	}

	return 0;
}

// Return number of iterations the GCF algorithm would take if not swapping n,m if n > m
int GCF_slow_iter(int m, int n)
{
	int r, iters = 0;
	while (r = m % n)
	{
		m = n;
		n = r;
		++iters;
	} 
	return iters;
}

// Find greatest common factor of a and b using Euclid's algorithm
// with modifications and optimizations by Knuth
int GCF(int m, int n)
{
	int r;
	int iters = 0;

	if (!n || !m)
		return 0;

	// Swap (n,m) if n > m (see Knuth, _The_Art_of_Computer_Programming_)
	// Without this condition, at least one more iteration is wasted!
	if (n > m)
	{
		int t = m;
		m = n;
		n = t;
	}

	// If n > m, GCF(m, n) = n
	// GCF(m, n) = GCF(m, r)
	while (r = m % n)
	{
		m = n;
		n = r;
		++iters;
	} 
	cout << "GCF iterations: " << iters;
	/* int wo_k = GCF_slow_iter(m, n);
	if (wo_k != iters)
		cout << " (" << wo_k << " without Knuth's optimizations)";*/

	cout << endl;
	return n;
}

// Return the number of decimal digits in n, useful for formatting
int decimal_digits(int n)
{
	int i = 0;

	// This algorithm comes from the Radix Conversion Formula
	while(n)
	{
		n /= 10;
		i++;
	}
	return i;
	
	/* Bad way to determine length of a number using C standard library functions
	char* lpsz = new char[n];
	sprintf(lpsz, "%d", n);
	i = strlen(lpsz);
	if (lpsz)
		free(lpsz);*/
}
