
#include <iostream.h>
#include <apvector.h>
#include <apstring.h>
#include <stdlib.h>
#include <fstream.h>
#include <ctype.h>

struct ADDRESS
{
	apstring name;
	apstring address;
	apstring city;
	apstring state;
	int zip;
	apstring phone;
};

istream& operator>>(istream& is, ADDRESS& addr);           // iostream operators
ostream& operator<<(ostream& os, const ADDRESS& addr);
istream& operator>>(istream& is, apvector<ADDRESS>& book);
ostream& operator<<(ostream& os, const apvector<ADDRESS>& book);

void save_book(apvector<ADDRESS>&);
void load_book(apvector<ADDRESS>&);
void backup();
void edit(apvector<ADDRESS>&, char*);
int end_of_file = 0;
int main(int argc, char** argv)
{
	apvector<ADDRESS> book;
	char* filename;

	cout << "Using address book " << filename << endl;

	load_book(book);

	while(1)
	{
		apstring choice;

		cout << "Address Book Program" << endl;
		cout << "A) Input (append)" << endl
			 << "V) View" << endl
			 << "S) Save and reload" << endl
			 << "L) Restore" << endl
			 << "C) Clear" << endl
			 << "X) Exit" << endl;
		getline(cin, choice);
		choice += "h";

		switch(tolower(choice[0]))
		{
		case 'x': exit(0); break;
		case 'a': cin >> book; break;
		case 'v': cout << book; break;
		case 's': save_book(book); break;
		case 'l': load_book(book); break;
		case 'c': book = 0; break;
		default: system("cls"); continue;
		}
		system("pause");
		system("cls");
	}

	return 0;
}


void save_book(apvector<ADDRESS>& b)
{	
	fstream fs("address.book", ios::out);
	if (!fs)
		cout << "could not open address.book";

	for(int i = 0; i < b.length(); i++)
	{
		fs   << b[i].name << endl
			 << b[i].address << endl
			 << b[i].city << endl
			 << b[i].state << endl
			 << b[i].zip << endl
			 << b[i].phone << endl;
	}

	cout << "book saved successfully to address.book" << endl;
	load_book(b);
}

void load_book(apvector<ADDRESS>& b)
{
	fstream fs("address.book", ios::in);
	if (!fs)
	{
		cout << "(failed to open file for reading)" << endl;
		return;
	}
	b = 0; // clear
	fs >> b;
	cout << "Successfully loaded " << b.length() << " entries from address.book" << endl;
}


istream& operator>>(istream& is, ADDRESS& addr)
{
	apstring temp;

	if (is == cin) cout << "Name [enter to quit]:    ";
	getline(is, addr.name); 
	if (!addr.name.length()) 
		end_of_file = 1;
 
	if (is == cin) cout << "Address [12345]:         ";
	getline(is, addr.address);
	if (!addr.address.length()) 
		addr.address = "12345 Noname Street";    // default street
	
	if (is == cin) cout << "City [Yucaipa]:          ";
	getline(is, addr.city);
	if (!addr.city.length())
		addr.city = "Yucaipa";                   // default city

	if (is == cin) cout << "State code [CA]:         ";
	getline(is, addr.state);
	
	if (!addr.state.length())
		addr.state = "CA";
	if (is == cin) cout << "Zip code [92399]:        ";
	getline(is, temp); 
	addr.zip = atoi(temp.c_str());
	if (!addr.zip)
		addr.zip = 92399;

	if (is == cin) cout << "Telephone [800-555-1212]: ";
	getline(is, addr.phone);
	if (!addr.phone.length())
		addr.phone = "800-555-1212";

	return is;
}

ostream& operator<<(ostream& os, const ADDRESS& addr)
{
	os << addr.name << endl
	   << "\t" << addr.address << endl
	   << "\t" << addr.city << ", " << addr.state << " " << addr.zip << endl
	   << "\t" << addr.phone << endl;
	return os;
}

// Input an entire book, appending to
istream& operator>>(istream& is, apvector<ADDRESS>& book)
{
	int i = book.length();
	book.resize(i + 1);

	while(1)
	{
		if (is == cin)
			cout << "entry #" << i + 1 << "--" << endl;
		is >> book[i++];
		if (end_of_file) {
			book.resize(i -= 1);
			break;
			end_of_file = 0;
		}
		// pause after every even entry
		if (i % 2) 
			system("pause");
		book.resize(i + 1);
	}
	return is;
}

// Output an entire book
ostream& operator<<(ostream& os, const apvector<ADDRESS>& book)
{
	for(int i = 0; i < book.length(); i++)
	{
		cout << i + 1 << ".\t" << book[i];
	}
	return os;
}
